from django.http import HttpResponseRedirect
from django.shortcuts import render, get_object_or_404
from unicom.models import Message, Chat
from unicom.services.crossplatform.send_message import send_message as send_message_generic
from django.contrib.auth.decorators import login_required


@login_required
def chat_history_view(request, chat_id):
    chat = get_object_or_404(Chat, id=chat_id)
    message_list = list(
        Message.objects
            .filter(chat_id=chat_id)
            .order_by('-timestamp')[:100]
    )[::-1]

    if request.method == 'POST':
        message_text = request.POST.get('message_text', '')
        if message_text.strip():
            send_message_generic({
                'platform': chat.platform,
                'chat_id': chat.id,
                'text': message_text,
            }, user=request.user)
            return HttpResponseRedirect(request.path_info)

    return render(request, 'admin/unicom/chat_history.html', {
        'chat': chat,
        'chat_messages_list': message_list,
        'without_messages': True  # This will be used to suppress the toast
    })



from django.apps import AppConfig
from unicom.services.telegram.set_telegram_webhook import set_telegram_webhook
from pprint import pprint


class UnicomConfig(AppConfig):
    name = 'unicom'

    def ready(self):
        import unicom.signals



from django.db import models
from .constants import channels


class Account(models.Model):
    id = models.CharField(max_length=500, primary_key=True)
    platform = models.CharField(max_length=100, choices=channels)
    is_bot = models.BooleanField(default=False)
    name = models.CharField(max_length=100, null=True, blank=True)
    # member = models.ForeignKey(
    #     'Member', on_delete=models.DO_NOTHING, null=True, blank=True)
    raw = models.JSONField()

    def __str__(self) -> str:
        return f"{self.platform}:{self.id} ({self.name})"

    # def get_menu(self):
    #     from robopower.models import Function
    #     # If self has a member, get the associated functions
    #     if self.member:
    #         member_functions = self.member.functions.all()
    #     else:
    #         member_functions = Function.objects.none()  # This returns an empty QuerySet

    #     # Get the public functions
    #     public_functions = Function.objects.filter(public=True)

    #     # Combine the two querysets
    #     combined_functions = member_functions | public_functions

    #     # Get unique function names and return
    #     return list(set(map(lambda f: f.name, combined_functions)))


from unicom.models import Message, AccountChat, Bot
from django.db import transaction
from django.db.models.signals import post_save, pre_save
from django.db import transaction
from django.dispatch import receiver
import threading


def handle_new_message(message):
    sender = message.sender
    # chat = message.chat
    # account_chat = AccountChat.objects.get(chat=chat, account=sender)
    print(f"New Message Received: {sender.name}: {message.text}")


@receiver(post_save, sender=Message)
def run_message_after_insert(sender, instance, created, **kwargs):
    if created:
        transaction.on_commit(lambda: threading.Thread(target=handle_new_message, args=(instance,)).start())


@receiver(pre_save, sender=Bot)
def bot_pre_save(sender, instance, **kwargs):
    # Save old config for comparison in post_save
    if instance.pk:
        try:
            old = sender.objects.get(pk=instance.pk)
            instance._old_config = old.config
        except sender.DoesNotExist:
            instance._old_config = None


@receiver(post_save, sender=Bot)
def run_bot_after_insert(sender, instance, created, **kwargs):
    # Check if created or config changed
    config_changed = not created and getattr(instance, '_old_config', None) != instance.config
    if created or config_changed:
        transaction.on_commit(lambda: threading.Thread(target=instance.validate).start())
 


from django.db import models
from django.contrib.auth.models import User
from unicom.models.constants import channels
from django.contrib.postgres.fields import ArrayField
from django.core.validators import validate_email
from fa2svg.converter import revert_to_original_fa


class Message(models.Model):
    TYPE_CHOICES = [
        ('text', 'Text'),
        ('html', 'HTML'),
        ('image', 'Image'),
        ('audio', 'Audio'),
    ]
    id = models.CharField(max_length=500, primary_key=True)
    platform = models.CharField(max_length=100, choices=channels)
    sender = models.ForeignKey('unicom.Account', on_delete=models.RESTRICT)
    user = models.ForeignKey(User, on_delete=models.RESTRICT, null=True, blank=True)
    chat = models.ForeignKey('unicom.Chat', on_delete=models.CASCADE, related_name='messages')
    is_bot = models.BooleanField(default=False)
    sender_name = models.CharField(max_length=100)
    subject = models.CharField(max_length=512, blank=True, null=True, help_text="Subject of the message (only for email messages)")
    text = models.TextField()
    html = models.TextField(
        blank=True, null=True,
        help_text="Full HTML body (only for email messages)"
    )
    to  = ArrayField(
        base_field=models.EmailField(validators=[validate_email]),
        blank=True,
        default=list,
        help_text="List of To: addresses",
    )
    cc  = ArrayField(
        base_field=models.EmailField(validators=[validate_email]),
        blank=True,
        default=list,
        help_text="List of Cc: addresses",
    )
    bcc = ArrayField(
        base_field=models.EmailField(validators=[validate_email]),
        blank=True,
        default=list,
        help_text="List of Bcc: addresses",
    )
    media = models.FileField(upload_to='media/', blank=True, null=True)
    reply_to_message = models.ForeignKey(
        'self', on_delete=models.SET_NULL, null=True, blank=True, related_name='replies')
    timestamp = models.DateTimeField()
    time_sent = models.DateTimeField(null=True, blank=True)
    time_delivered = models.DateTimeField(null=True, blank=True)
    time_seen = models.DateTimeField(null=True, blank=True)
    sent = models.BooleanField(default=False)
    delivered = models.BooleanField(default=False)
    seen = models.BooleanField(default=False)
    raw = models.JSONField()
    media_type = models.CharField(
        max_length=10,
        choices=TYPE_CHOICES,
        default='text'
    )
    # Add any additional fields as needed for specific platforms

    @property
    def original_content(self):
        return revert_to_original_fa(self.html) if self.platform == 'Email' else self.text

    class Meta:
        ordering = ['-timestamp']

    def __str__(self) -> str:
        return f"{self.platform}:{self.chat_id}:{self.id}->{self.sender_id}({self.sender_name}): {self.text}"



from django.db import models
from .constants import channels


class Chat(models.Model):
    id = models.CharField(max_length=500, primary_key=True)
    platform = models.CharField(max_length=100, choices=channels)
    is_private = models.BooleanField(default=True)
    name = models.CharField(max_length=100, null=True, blank=True)
    # accounts = models.ManyToManyField('unicom.Account', related_name="chats")

    def __str__(self) -> str:
        return f"{self.platform}:{self.id} ({self.name})"


from django.contrib import admin
from unicom.models import Message, Update, Chat, Account, AccountChat, Bot
from django.utils.html import format_html
from unicom.views.chat_history_view import chat_history_view
from django.urls import path


class ChatAdmin(admin.ModelAdmin):
    list_filter = ('platform', 'is_private')
    list_display = ('id', 'name', 'view_chat_link')
    search_fields = ('id', 'name') 

    def view_chat_link(self, obj):
        return format_html('<a href="{}" target="_blank">View Chat</a>', self.url_for_chat(obj.id))

    def url_for_chat(self, id):
        return f"{id}/messages/"

    def get_urls(self):
        urls = super().get_urls()
        custom_urls = [
            path('<path:chat_id>/messages/', self.admin_site.admin_view(chat_history_view), name='chat-detail')
        ]
        return custom_urls + urls

    view_chat_link.short_description = 'View Chat'


class AccountChatAdmin(admin.ModelAdmin):
    list_filter = ('account__platform', )
    search_fields = ('account__name', 'chat__name') 


class AccountAdmin(admin.ModelAdmin):
    list_filter = ('platform', )
    search_fields = ('name', )

class BotAdmin(admin.ModelAdmin):
    list_filter = ('platform', )
    search_fields = ('name', )
    list_display = ('id', 'name', 'platform', 'active', 'confirmed_webhook_url', 'error')

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return ['active', 'confirmed_webhook_url', 'error']
        return super().get_readonly_fields(request, obj)

admin.site.register(Bot)
admin.site.register(Message)
admin.site.register(Update)
admin.site.register(Chat, ChatAdmin)
admin.site.register(Account, AccountAdmin)
admin.site.register(AccountChat, AccountChatAdmin)



# Generated by Django 4.2.20 on 2025-05-20 11:04

from django.conf import settings
import django.contrib.postgres.fields
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.CharField(max_length=500, primary_key=True, serialize=False)),
                ('platform', models.CharField(choices=[('Telegram', 'Telegram'), ('WhatsApp', 'WhatsApp'), ('Internal', 'Internal'), ('Email', 'Email')], max_length=100)),
                ('is_bot', models.BooleanField(default=False)),
                ('name', models.CharField(blank=True, max_length=100, null=True)),
                ('raw', models.JSONField()),
            ],
        ),
        migrations.CreateModel(
            name='Bot',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=100)),
                ('platform', models.CharField(choices=[('Telegram', 'Telegram'), ('WhatsApp', 'WhatsApp'), ('Internal', 'Internal'), ('Email', 'Email')], max_length=100)),
                ('config', models.JSONField()),
                ('active', models.BooleanField(default=False, editable=False)),
                ('confirmed_webhook_url', models.CharField(blank=True, editable=False, max_length=500, null=True)),
                ('error', models.CharField(blank=True, editable=False, max_length=500, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Chat',
            fields=[
                ('id', models.CharField(max_length=500, primary_key=True, serialize=False)),
                ('platform', models.CharField(choices=[('Telegram', 'Telegram'), ('WhatsApp', 'WhatsApp'), ('Internal', 'Internal'), ('Email', 'Email')], max_length=100)),
                ('is_private', models.BooleanField(default=True)),
                ('name', models.CharField(blank=True, max_length=100, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.CharField(max_length=500, primary_key=True, serialize=False)),
                ('platform', models.CharField(choices=[('Telegram', 'Telegram'), ('WhatsApp', 'WhatsApp'), ('Internal', 'Internal'), ('Email', 'Email')], max_length=100)),
                ('is_bot', models.BooleanField(default=False)),
                ('sender_name', models.CharField(max_length=100)),
                ('subject', models.CharField(blank=True, help_text='Subject of the message (only for email messages)', max_length=512, null=True)),
                ('text', models.TextField()),
                ('html', models.TextField(blank=True, help_text='Full HTML body (only for email messages)', null=True)),
                ('to', django.contrib.postgres.fields.ArrayField(base_field=models.EmailField(max_length=254, validators=[django.core.validators.EmailValidator()]), blank=True, default=list, help_text='List of To: addresses', size=None)),
                ('cc', django.contrib.postgres.fields.ArrayField(base_field=models.EmailField(max_length=254, validators=[django.core.validators.EmailValidator()]), blank=True, default=list, help_text='List of Cc: addresses', size=None)),
                ('bcc', django.contrib.postgres.fields.ArrayField(base_field=models.EmailField(max_length=254, validators=[django.core.validators.EmailValidator()]), blank=True, default=list, help_text='List of Bcc: addresses', size=None)),
                ('media', models.FileField(blank=True, null=True, upload_to='media/')),
                ('timestamp', models.DateTimeField()),
                ('time_sent', models.DateTimeField(blank=True, null=True)),
                ('time_delivered', models.DateTimeField(blank=True, null=True)),
                ('time_seen', models.DateTimeField(blank=True, null=True)),
                ('sent', models.BooleanField(default=False)),
                ('delivered', models.BooleanField(default=False)),
                ('seen', models.BooleanField(default=False)),
                ('raw', models.JSONField()),
                ('media_type', models.CharField(choices=[('text', 'Text'), ('html', 'HTML'), ('image', 'Image'), ('audio', 'Audio')], default='text', max_length=10)),
                ('chat', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='messages', to='unicom.chat')),
                ('reply_to_message', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='replies', to='unicom.message')),
                ('sender', models.ForeignKey(on_delete=django.db.models.deletion.RESTRICT, to='unicom.account')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.RESTRICT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['-timestamp'],
            },
        ),
        migrations.CreateModel(
            name='Update',
            fields=[
                ('platform', models.CharField(choices=[('Telegram', 'Telegram'), ('WhatsApp', 'WhatsApp'), ('Internal', 'Internal'), ('Email', 'Email')], max_length=100)),
                ('id', models.CharField(max_length=100, primary_key=True, serialize=False)),
                ('payload', models.JSONField()),
                ('bot', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='unicom.bot')),
                ('message', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='unicom.message')),
            ],
        ),
        migrations.CreateModel(
            name='AccountChat',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('account', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='unicom.account')),
                ('chat', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='unicom.chat')),
            ],
            options={
                'unique_together': {('account', 'chat')},
            },
        ),
    ]



from django.urls import path
from unicom.views.telegram_webhook import telegram_webhook
from unicom.views.whatsapp_webhook import whatsapp_webhook

urlpatterns = [
    path('telegram/<int:bot_id>', telegram_webhook),
    path('whatsapp', whatsapp_webhook),
]
